#!/usr/bin/env python
# Copyright (c) 2009 Sebastian Reichel <elektranox@gmail.com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

# User Information
LOCATION="Otto-Wels-Str. | OL"
NICKNAME="elektranox"
EMAIL="elektranox@gmail.com"

# Freifunk Information
FREIFUNK_NAME="Freifunk Oldenburg"
FREIFUNK_URL="http://oldenburg.freifunk.net"
FREIFUNK_PREFIX="10.18"
FREIFUNK_SSID="oldenburg.freifunk.net"

POS_LAT="53.09464899290893"
POS_LON="8.216689825057983"

# TXT INFO is reachable @
TXTINFO="http://127.0.0.1:2006"

# Interfaces
INTERFACES=["tap0", "ath1"]

# ------------------------------------------------------------------------

import os, sys, re, urllib
def parse(data, i=1):
	output = ""
	if type(data) == type("str"):
		output = "\""+data+"\""
	elif type(data) == type(123) or type(data) == type(2.3):
		output = str(data)
	elif type(data) == type(True):
		output = "true" if data==True else "false"
	elif type(data) == type({}):
		for k,v in data.items():
			output += i*"\t"+"\""+k+"\":"+parse(v, i+1)+",\n"
		if output != "":
			output = output[:-2]+"\n" # Remote comma of last entry
		if i!=1: # 1 = Basis Layer
			output = "{\n"+output+(i-1)*"\t"+"}"
	elif type(data) == type([]):
		for entry in data:
			output += i*"\t"+parse(entry, i+1)+",\n"
		if output != "":
			output = output[:-2]+"\n" # Remove comma of last entry
		output = "[\n"+output+(i-1)*"\t"+"]"
	else:
		# print "ERROR: Type not supported: ", type(data)
		output = ""
	return output

def parse_section(data, comma=True):
	return parse(data)[:-1] + "," if comma else parse(data)[:-1]

def section_freifunk():
	return {
		"freifunk" : {
			"contact" : {
				".name"      : "contact",
				".type"      : "public",
				"location"   : LOCATION,
				"mail"       : EMAIL,
				"nickname"   : NICKNAME,
				".anonymous" : False,
				".index"     : 1
			},
			"community" : {
				".name"      : "community",
				".type"      : "public",
				"name"       : FREIFUNK_NAME,
				"homepage"   : FREIFUNK_URL,
				"prefix"     : FREIFUNK_PREFIX,
				"ssid"       : FREIFUNK_SSID,
				".anonymous" : False,
				".index"     : 2
			}
		}
	}

def section_geo():
	return {
		"geo": {
			"latitude"  : POS_LAT,
			"longitude" : POS_LON
		}
	}

def section_protocol():
	return {"protocol":1}

def section_wireless():
	return {
		"wireless" : {
			"interfaces": [],
			"status" : [],
			"devices" : {}
		}
	}

def section_olsrd():
	counter=0
	links=[]; neighbors=[]; topology=[]; hna=[]; mid=[]; routes=[];
	for line in urllib.urlopen(TXTINFO).readlines():
		counter+=1
		data = line.split()
		if line == "\n" or counter<=0:
			pass
		elif "Table: " in line:
			section = line[7:-1]
			counter=-1
			pass
		elif section == "Links":
			links += [{"Local IP":data[0], "Remote IP":data[1], "Hyst.":data[2], "Cost":data[4], "LQ":data[3][:5], "NLQ":data[3][6:]}]
		elif section == "Neighbors":
			neighbors += [{"IP address":data[0], "SYM":data[1], "MPR":data[2], "MPRS":data[3], "Will.":data[4], "2 Hop Neighbors":data[5]}]
		elif section == "Topology":
			topology += [{"Dest. IP":data[0], "Last hop IP":data[1], "Cost":data[3], "LQ":data[2][:5], "NLQ":data[2][6:]}]
		elif section == "HNA":
			hna += [{"Gateway":data[2], "Destination":data[0]+"/"+data[1]}]
		elif section == "MID":
			try:
				mid += [{"IP address":data[0], "Aliases":data[1]}]
			except IndexError:
				mid += [{"IP address":data[0], "Aliases":""}]
		elif section == "Routes":
			routes += [{"Destination":data[0], "Gateway IP":data[1], "Metric":data[2], "ETX":data[3], "Interface":data[4]}]
			
	return {
		"olsrd" : {
			"Routes" : routes,
			"HNA": hna,
			"MID": mid,
			"Topology": topology,
			"Neighbors": neighbors,
			"Links": links
		}
	}

def section_network():
	return {
		"network" : {
			"ovpn" : {},
			"wlan" : {},
		}
	}

def section_firmware():
	return {
		"firmware" : {
			"luciversion" : "0.0.0",
			"luciname" : "elektranox's luci status replacement",
			"distversion" : "Lenny (5.0)",	# FIXME: do not hardcode this information
			"distname" : "Debian"			# FIXME: do not hardcode this information
		}
	}

def section_system():
	modelname = ""
	for line in open("/proc/cpuinfo").readlines():
		if "model name" in line:
			modelname = line[13:-1]
			break
	for line in open("/proc/meminfo").readlines():
		if "MemTotal:" in line:
			memtotal = int(line[16:-4])
		elif "Cached:" in line:
			memcached = int(line[18:-4])
		elif "MemFree:" in line:
			memfree = int(line[17:-4])
		elif "Buffers:" in line:
			membuffers = int(line[17:-4])
	return {
		"system" : {
			"hostname" : os.uname()[1],
			"sysinfo" : [os.popen("uname -m").read()[:-1], modelname, memtotal, memcached, membuffers, memfree],
			"uptime" : [str(field) for field in open("/proc/uptime").read().split()],
			"loadavg" : [str(field) for field in open("/proc/loadavg").read().split()]
		}
	}

def index():
	output = "{\n"
	output += parse_section(section_freifunk())+"\n"
	output += parse_section(section_protocol())+"\n"
	output += parse_section(section_wireless())+"\n" # TODO
	output += parse_section(section_olsrd())+"\n"
	output += parse_section(section_network())+"\n"  # TODO
	output += parse_section(section_firmware())+"\n"
	output += parse_section(section_geo())+"\n"
	output += parse_section(section_system(), False)+"\n"
	output += "}"
	return output

def cgi():
	import cgi
	print "Content-type: text\n"
	sys.stdout.flush()

cgi()
print index()